@@ -37,18 +37,10 @@
 #include <eel/eel-gtk-macros.h>
 #include <gdk/gdkcursor.h>
 #include <gdk/gdkkeysyms.h>
-#include <gtk/gtkdialog.h>
-#include <gtk/gtkbindings.h>
-#include <gtk/gtkcellrendererpixbuf.h>
-#include <gtk/gtkcellrenderertext.h>
-#include <gtk/gtkentry.h>
-#include <gtk/gtklabel.h>
-#include <gtk/gtkvbox.h>
-#include <gtk/gtkstock.h>
-#include <gtk/gtktreeselection.h>
-#include <gtk/gtktreeview.h>
+#include <gtk/gtk.h>
 #include <libegg/eggtreemultidnd.h>
 #include <glib/gi18n.h>
+#include <glib-object.h>
 #include <libgnome/gnome-macros.h>
 #include <libnautilus-extension/nautilus-column-provider.h>
 #include <libnautilus-private/nautilus-column-chooser.h>
@@ -112,6 +104,8 @@ struct FMListViewDetails {
 	NautilusFile *renaming_file;
 	gboolean rename_done;
 	guint renaming_file_activate_timeout;
+
+	GQuark last_sort_attr;
 };
 
 struct SelectionForeachData {
@@ -230,17 +224,27 @@ activate_selected_items (FMListView *view)
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (view),
 					  file_list,
 					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-					  0);
+					  0,
+					  TRUE);
 	nautilus_file_list_free (file_list);
 
 }
 
 static void
 activate_selected_items_alternate (FMListView *view,
-				   NautilusFile *file)
+				   NautilusFile *file,
+				   gboolean open_in_tab)
 {
 	GList *file_list;
+	NautilusWindowOpenFlags flags;
 
+	flags = NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND;
+
+	if (open_in_tab && eel_preferences_get_boolean (NAUTILUS_PREFERENCES_ENABLE_TABS)) {
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_TAB;
+        } else {
+		flags |= NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW;
+        }
 
 	if (file != NULL) {
 		nautilus_file_ref (file);
@@ -251,8 +255,8 @@ activate_selected_items_alternate (FMListView *view,
 	fm_directory_view_activate_files (FM_DIRECTORY_VIEW (view),
 					  file_list,
 					  NAUTILUS_WINDOW_OPEN_ACCORDING_TO_MODE,
-					  NAUTILUS_WINDOW_OPEN_FLAG_CLOSE_BEHIND |
-					  NAUTILUS_WINDOW_OPEN_FLAG_NEW_WINDOW);
+					  flags,
+					  TRUE);
 	nautilus_file_list_free (file_list);
 
 }
@@ -293,7 +297,7 @@ fm_list_view_did_not_drag (FMListView *view,
 			if (event->button == 1) {
 				activate_selected_items (view);
 			} else if (event->button == 2) {
-				activate_selected_items_alternate (view, NULL);
+				activate_selected_items_alternate (view, NULL, TRUE);
 			}
 		}
 		gtk_tree_path_free (path);
@@ -661,14 +665,14 @@ button_press_callback (GtkWidget *widget, GdkEventButton *event, gpointer callba
 					if ((event->button == 1 || event->button == 3)) {
 						activate_selected_items (view);
 					} else if (event->button == 2) {
-						activate_selected_items_alternate (view, NULL);
+						activate_selected_items_alternate (view, NULL, TRUE);
 					}
 				} else if (event->button == 1 &&
 					   (event->state & GDK_SHIFT_MASK) != 0) {
 					NautilusFile *file;
 					file = fm_list_model_file_for_path (view->details->model, path);
 					if (file != NULL) {
-						activate_selected_items_alternate (view, file);
+						activate_selected_items_alternate (view, file, TRUE);
 						nautilus_file_unref (file);
 					}
 				}
@@ -951,7 +955,7 @@ key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_dat
 			break;
 		}
 		if ((event->state & GDK_SHIFT_MASK) != 0) {
-			activate_selected_items_alternate (FM_LIST_VIEW (view), NULL);
+			activate_selected_items_alternate (FM_LIST_VIEW (view), NULL, TRUE);
 		} else {
 			activate_selected_items (FM_LIST_VIEW (view));
 		}
@@ -960,7 +964,7 @@ key_press_callback (GtkWidget *widget, GdkEventKey *event, gpointer callback_dat
 	case GDK_Return:
 	case GDK_KP_Enter:
 		if ((event->state & GDK_SHIFT_MASK) != 0) {
-			activate_selected_items_alternate (FM_LIST_VIEW (view), NULL);
+			activate_selected_items_alternate (FM_LIST_VIEW (view), NULL, TRUE);
 		} else {
 			activate_selected_items (FM_LIST_VIEW (view));
 		}
@@ -1004,12 +1008,34 @@ fm_list_view_reveal_selection (FMDirectoryView *view)
         nautilus_file_list_free (selection);
 }
 
+static gboolean
+sort_criterion_changes_due_to_user (GtkTreeView *tree_view)
+{
+	GList *columns, *p;
+	GtkTreeViewColumn *column;
+	GSignalInvocationHint *ihint;
+	unsigned int sort_signal_id;
+
+	sort_signal_id = g_signal_lookup ("clicked", gtk_tree_view_column_get_type ());
+
+	columns = gtk_tree_view_get_columns (tree_view);
+	for (p = columns; p != NULL; p = p->next) {
+		column = p->data;
+		ihint = g_signal_get_invocation_hint (column);
+		if (ihint != NULL) {
+			return TRUE;
+		}
+	}
+
+	return FALSE;
+}
+
 static void
 sort_column_changed_callback (GtkTreeSortable *sortable, 
 			      FMListView *view)
 {
 	NautilusFile *file;
-	gint sort_column_id;
+	gint sort_column_id, default_sort_column_id;
 	GtkSortType reversed;
 	GQuark sort_attr, default_sort_attr;
 	char *reversed_attr, *default_reversed_attr;
@@ -1017,21 +1043,46 @@ sort_column_changed_callback (GtkTreeSortable *sortable,
 	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view));
 
 	gtk_tree_sortable_get_sort_column_id (sortable, &sort_column_id, &reversed);
-
 	sort_attr = fm_list_model_get_attribute_from_sort_column_id (view->details->model, sort_column_id);
-	sort_column_id = fm_list_model_get_sort_column_id_from_attribute (view->details->model,
-									  g_quark_from_string (default_sort_order_auto_value));
-	default_sort_attr = fm_list_model_get_attribute_from_sort_column_id (view->details->model, sort_column_id);
+
+	default_sort_column_id = fm_list_model_get_sort_column_id_from_attribute (view->details->model,
+										  g_quark_from_string (default_sort_order_auto_value));
+	default_sort_attr = fm_list_model_get_attribute_from_sort_column_id (view->details->model, default_sort_column_id);
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_COLUMN,
 				    g_quark_to_string (default_sort_attr), g_quark_to_string (sort_attr));
 
 	default_reversed_attr = (default_sort_reversed_auto_value ? "true" : "false");
+
+	if (view->details->last_sort_attr != sort_attr &&
+	    sort_criterion_changes_due_to_user (view->details->tree_view)) {
+		/* at this point, the sort order is always GTK_SORT_ASCENDING, if the sort column ID
+		 * switched. Invert the sort order, if it's the default criterion with a reversed preference,
+		 * or if it makes sense for the attribute (i.e. date). */
+		if (sort_attr == default_sort_attr) {
+			/* use value from preferences */
+			reversed = eel_preferences_get_boolean (NAUTILUS_PREFERENCES_LIST_VIEW_DEFAULT_SORT_IN_REVERSE_ORDER);
+		} else {
+			reversed = nautilus_file_is_date_sort_attribute_q (sort_attr);
+		}
+
+		if (reversed) {
+			g_signal_handlers_block_by_func (sortable, sort_column_changed_callback, view);
+			gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (view->details->model),
+							      sort_column_id,
+							      GTK_SORT_DESCENDING);
+			g_signal_handlers_unblock_by_func (sortable, sort_column_changed_callback, view);
+		}
+	}
+
+
 	reversed_attr = (reversed ? "true" : "false");
 	nautilus_file_set_metadata (file, NAUTILUS_METADATA_KEY_LIST_VIEW_SORT_REVERSED,
 				    default_reversed_attr, reversed_attr);
 				    
 	/* Make sure selected item(s) is visible after sort */
 	fm_list_view_reveal_selection (FM_DIRECTORY_VIEW (view));							      
+
+	view->details->last_sort_attr = sort_attr;
 }
 
 static void
@@ -1152,12 +1203,17 @@ move_copy_items_callback (NautilusTreeViewDragDest *dest,
 			  gpointer user_data)
 
 {
+	FMDirectoryView *view = user_data;
+
+	nautilus_clipboard_clear_if_colliding_uris (GTK_WIDGET (view),
+						    item_uris,
+						    fm_directory_view_get_copied_files_atom (view));
 	fm_directory_view_move_copy_items (item_uris,
 					   NULL,
 					   target_uri,
 					   action,
 					   x, y,
-					   FM_DIRECTORY_VIEW (user_data));
+					   view);
 }
 
 static void
@@ -1926,6 +1982,43 @@ fm_list_view_set_selection (FMDirectoryView *view, GList *selection)
 	fm_directory_view_notify_selection_changed (view);
 }
 
+static void
+fm_list_view_invert_selection (FMDirectoryView *view)
+{
+	FMListView *list_view;
+	GtkTreeSelection *tree_selection;
+	GList *node;
+	GList *iters, *l;
+	NautilusFile *file;
+	GList *selection = NULL;
+	
+	list_view = FM_LIST_VIEW (view);
+	tree_selection = gtk_tree_view_get_selection (list_view->details->tree_view);
+
+	g_signal_handlers_block_by_func (tree_selection, list_selection_changed_callback, view);
+	
+	gtk_tree_selection_selected_foreach (tree_selection,
+					 fm_list_view_get_selection_foreach_func, &selection);
+
+	gtk_tree_selection_select_all (tree_selection);
+	
+	for (node = selection; node != NULL; node = node->next) {
+		file = node->data;
+		iters = fm_list_model_get_all_iters_for_file (list_view->details->model, file);
+
+		for (l = iters; l != NULL; l = l->next) {
+			gtk_tree_selection_unselect_iter (tree_selection,
+							(GtkTreeIter *)l->data);
+		}
+		eel_g_list_free_deep (iters);
+	}
+
+	g_list_free (selection);
+
+	g_signal_handlers_unblock_by_func (tree_selection, list_selection_changed_callback, view);
+	fm_directory_view_notify_selection_changed (view);
+}
+
 static void
 fm_list_view_select_all (FMDirectoryView *view)
 {
@@ -2031,39 +2124,53 @@ create_column_editor (FMListView *view)
 	GtkWidget *label;
 	GtkWidget *box;
 	GtkWidget *column_chooser;
+	GtkWidget *alignment;
 	NautilusFile *file;
-	char *title;
+	char *str;
 	char *name;
+	const char *label_text;
 	
 	file = fm_directory_view_get_directory_as_file (FM_DIRECTORY_VIEW (view));
 	name = nautilus_file_get_display_name (file);
-	title = g_strdup_printf (_("%s Visible Columns"), name);
+	str = g_strdup_printf (_("%s Visible Columns"), name);
 	g_free (name);
 
-	window = gtk_dialog_new_with_buttons (title, 
+	window = gtk_dialog_new_with_buttons (str,
 					      GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view))),
 					      GTK_DIALOG_DESTROY_WITH_PARENT,
-					      GTK_STOCK_CLOSE, GTK_RESPONSE_CANCEL,
+					      GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 					      NULL);
-	g_free (title);
+	g_free (str);
 	g_signal_connect (window, "response", 
 			  G_CALLBACK (column_editor_response_callback), NULL);
 	
 	gtk_window_set_default_size (GTK_WINDOW (window), 300, 400);
 
-	box = gtk_vbox_new (FALSE, 6);
+	box = gtk_vbox_new (FALSE, 12);
 	gtk_container_set_border_width (GTK_CONTAINER (box), 12);
 	gtk_widget_show (box);
 	gtk_container_add (GTK_CONTAINER (GTK_DIALOG (window)->vbox), box);
 
-	label = gtk_label_new (_("Choose the order of information to appear in this folder."));
-	gtk_label_set_line_wrap (GTK_LABEL (label), TRUE);
+	label_text = _("Choose the order of information to appear in this folder:");
+	str = g_strconcat ("<b>", label_text, "</b>", NULL);
+	label = gtk_label_new (NULL);
+	gtk_label_set_markup (GTK_LABEL (label), str);
+	gtk_label_set_line_wrap (GTK_LABEL (label), FALSE);
+	gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
 	gtk_widget_show (label);
 	gtk_box_pack_start (GTK_BOX (box), label, FALSE, FALSE, 0);
 
+	g_free (str);
+
+	alignment = gtk_alignment_new (0.5, 0.5, 1, 1);
+	gtk_alignment_set_padding (GTK_ALIGNMENT (alignment),
+				   0, 0, 12, 0);
+	gtk_widget_show (alignment);
+	gtk_box_pack_start (GTK_BOX (box), alignment, TRUE, TRUE, 0);
+
 	column_chooser = nautilus_column_chooser_new ();
 	gtk_widget_show (column_chooser);
-	gtk_box_pack_start (GTK_BOX (box), column_chooser, TRUE, TRUE, 0);
+	gtk_container_add (GTK_CONTAINER (alignment), column_chooser);
 
 	g_signal_connect (column_chooser, "changed",
 			  G_CALLBACK (column_chooser_changed_callback),
@@ -2133,6 +2240,24 @@ fm_list_view_merge_menus (FMDirectoryView *view)
 	list_view->details->menus_ready = TRUE;
 }
 
+static void
+fm_list_view_unmerge_menus (FMDirectoryView *view)
+{
+	FMListView *list_view;
+	GtkUIManager *ui_manager;
+
+	list_view = FM_LIST_VIEW (view);
+
+	FM_DIRECTORY_VIEW_CLASS (fm_list_view_parent_class)->unmerge_menus (view);
+
+	ui_manager = fm_directory_view_get_ui_manager (view);
+	if (ui_manager != NULL) {
+		nautilus_ui_unmerge_ui (ui_manager,
+					&list_view->details->list_merge_id,
+					&list_view->details->list_action_group);
+	}
+}
+
 static void
 fm_list_view_update_menus (FMDirectoryView *view)
 {
@@ -2509,7 +2634,6 @@ static void
 fm_list_view_dispose (GObject *object)
 {
 	FMListView *list_view;
-	GtkUIManager *ui_manager;
 
 	list_view = FM_LIST_VIEW (object);
 
@@ -2529,13 +2653,6 @@ fm_list_view_dispose (GObject *object)
 		list_view->details->renaming_file_activate_timeout = 0;
 	}
 
-	ui_manager = fm_directory_view_get_ui_manager (FM_DIRECTORY_VIEW (list_view));
-	if (ui_manager != NULL) {
-		nautilus_ui_unmerge_ui (ui_manager,
-					&list_view->details->list_merge_id,
-					&list_view->details->list_action_group);
-	}
-
 	G_OBJECT_CLASS (parent_class)->dispose (object);
 }
 
@@ -2690,12 +2807,14 @@ fm_list_view_class_init (FMListViewClass *class)
 	fm_directory_view_class->is_empty = fm_list_view_is_empty;
 	fm_directory_view_class->remove_file = fm_list_view_remove_file;
 	fm_directory_view_class->merge_menus = fm_list_view_merge_menus;
+	fm_directory_view_class->unmerge_menus = fm_list_view_unmerge_menus;
 	fm_directory_view_class->update_menus = fm_list_view_update_menus;
 	fm_directory_view_class->reset_to_defaults = fm_list_view_reset_to_defaults;
 	fm_directory_view_class->restore_default_zoom_level = fm_list_view_restore_default_zoom_level;
 	fm_directory_view_class->reveal_selection = fm_list_view_reveal_selection;
 	fm_directory_view_class->select_all = fm_list_view_select_all;
 	fm_directory_view_class->set_selection = fm_list_view_set_selection;
+	fm_directory_view_class->invert_selection = fm_list_view_invert_selection;
 	fm_directory_view_class->compare_files = fm_list_view_compare_files;
 	fm_directory_view_class->sort_directories_first_changed = fm_list_view_sort_directories_first_changed;
 	fm_directory_view_class->start_renaming_file = fm_list_view_start_renaming_file;
@@ -2773,11 +2892,13 @@ fm_list_view_init (FMListView *list_view)
 }
 
 static NautilusView *
-fm_list_view_create (NautilusWindowInfo *window)
+fm_list_view_create (NautilusWindowSlotInfo *slot)
 {
 	FMListView *view;
 
-	view = g_object_new (FM_TYPE_LIST_VIEW, "window", window, NULL);
+	view = g_object_new (FM_TYPE_LIST_VIEW,
+			     "window-slot", slot,
+			     NULL);
 	g_object_ref (view);
 	gtk_object_sink (GTK_OBJECT (view));
 	return NAUTILUS_VIEW (view);
@@ -2806,9 +2927,11 @@ fm_list_view_supports_uri (const char *uri,
 
 static NautilusViewInfo fm_list_view = {
 	FM_LIST_VIEW_ID,
-	N_("List"),
-	N_("View as List"),
-	N_("View as _List"),
+	/* translators: this is used in the view selection dropdown
+	 * of navigation windows and in the preferences dialog */
+	N_("List View"),
+	/* translators: this is used in the view menu */
+	N_("_List"),
 	N_("The list view encountered an error."),
 	N_("The list view encountered an error while starting up."),
 	N_("Display this location with the list view."),
@@ -2819,9 +2942,8 @@ static NautilusViewInfo fm_list_view = {
 void
 fm_list_view_register (void)
 {
-	fm_list_view.label = _(fm_list_view.label);
-	fm_list_view.view_as_label = _(fm_list_view.view_as_label);
-	fm_list_view.view_as_label_with_mnemonic = _(fm_list_view.view_as_label_with_mnemonic);
+	fm_list_view.view_combo_label = _(fm_list_view.view_combo_label);
+	fm_list_view.view_menu_label_with_mnemonic = _(fm_list_view.view_menu_label_with_mnemonic);
 	fm_list_view.error_label = _(fm_list_view.error_label);
 	fm_list_view.startup_error_label = _(fm_list_view.startup_error_label);
 	fm_list_view.display_location_label = _(fm_list_view.display_location_label);
